<?php defined( '_JEXEC' ) or die( 'Restricted access' );

/**
 * @version 2.0 $Id: links.php,v 1.7 2010-04-06 23:43:46 harry Exp $
 * @package Joomla
 * @subpackage H2N Link Directory 2
 * @copyright (C) 2008 - 2009 Harry Klein - www.joomla-hklein.de
 * @Copyright (C) 2006 Soner (pisdoktor) Ekici - www.sonerekici.com
 * @license GNU/GPL, see LICENSE.php
 * 
 * H2N Link Directory 2 is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 2
 * as published by the Free Software Foundation.
 *
 * H2N Link Directory 2 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with EventList; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

jimport( 'joomla.application.component.model' );

require_once(JPATH_COMPONENT_ADMINISTRATOR.DS.'LDConfig.php');
require_once(JPATH_COMPONENT.DS.'classes'.DS.'LinkdirectoryURL.php');

class LinkDirectory2ModelLinks extends JModel {

	private $_ld;
	private $_rows_of_links;
	private $_total;
	private $_paginationLinks;
	private $_searchtext;
	
	public function __construct(){
		parent::__construct();
		$this->_ld=new LDConfig();
	}

	public function getSearchResult(){
		$this->_searchtext = JRequest::getString( 'searchtext', 0, 'get' );
		$catid = JRequest::getInt( 'catid', 0, 'get' );
		$this->search($this->_searchtext,$catid);
		return $this->_rows_of_links;
	}
	
	public function getSearchText(){
		return $this->_searchtext;
	}
	
	public function getTotal(){
		return $this->_total;
	}
	
	public function &getPageNavigation() {
		if(!$this->_paginationLinks) {
			jimport('joomla.html.pagination');
			$this->_paginationLinks = new JPagination($this->getTotal(), JRequest::getVar('limitstart', 0),  JRequest::getVar('limit', $this->_ld->limitlink));
		}
		return $this->_paginationLinks;
	}	

	/**
	 * Suche nach Links. Es wird nach title, url, description, long_description (abhängig von der Konfigurration)
	 * der Links gesucht. Die jeweiligen Link müssen veröffentlicht sein, ebenso die referenzierten Kategorien.
	 * 
	 * Abweichend vom üblichen wird nur ein Subset vom (Such)Ergebnis geliefert, weil das Ergebnis sehr 
	 * groß sein kann. Die Auswahl des Subset via limitstart (aus Request) und limitlink (aus der H2N Link Directory Konfig)
	 * erfolgt. 
	 *
	 * @param string $searchtext Text, nach dem gesucht werden soll
	 * @param int $catid Kategorieid, unterhalb der die Suche erfolgen soll
	 * @return Array mit dem Suchergebnis,
	 */
	private function search($searchtext, $catid) {
		$db = &JFactory::getDBO();
		$searchtext = $db->getEscaped( trim( strtolower( $searchtext ) ) );
		$limitstart = JRequest::getInt( 'limitstart', 0, 'get' );

		if ($searchtext) {
			$where = array();
			$where[] = '#__ldlinks.published = 1';
			$this->getWhereAllSubCategoriesForCategory($catid, $where);
			$this->getWhereSearchtextForLink($searchtext, $where);
			$whereAsString = count( $where ) ? "\n WHERE " . implode( ' AND ', $where ) : "" ;
			
			$queryForSearch = 'SELECT #__ldlinks.* , #__ldcats.name as category_name FROM #__ldlinks '
			. 'LEFT OUTER JOIN #__ldcats ON #__ldlinks.catid = #__ldcats.id AND #__ldcats.published = 1 '  
			. $whereAsString
			. ' ORDER BY ' . $this->_ld->linkorder 
			;
			$this->_total = $this->_getListCount($queryForSearch);
			$query = $queryForSearch . " LIMIT $limitstart,". $this->_ld->limitlink ;
			$db->setQuery($query);
			$this->_rows_of_links = $db->loadObjectList();
		} else {
			$this->_rows_of_links=NULL;
			$this->_total = 0;
		}
		return $this->_rows_of_links;
	}
	
	/**
	 * Liefert where-Klausel, um Links in der aktuellen und deren Unterkategorien suchen zu können.
	 *
	 * @param int $catid Katehorie-Id
	 * @param Array mit $where-Klausel, deren Elemente später mit AND verknüpft werden
	 * @global ld->searchonlyinsubcats
	 * @see #getWhereAllSubCategoriesFor
	 *
	 */
	private function getWhereAllSubCategoriesForCategory($catid,&$where){
		require_once(JPATH_COMPONENT.DS.'classes'.DS.'SearchUtils.php');
		SearchUtils::getWhereAllSubCategoriesFor($catid, 'catid', $where);
	}

	/**
	 * Baut die where-Klause bezüglich des Suchtextes für die Suche nach Links zusammen.
	 * Dabei wird berücksichtigt, nach welchen Feldern (title,url,description, long_description) 
	 * eines Links gesucht werden soll.
	 *
	 * @param String $searchtext Suchtext
	 * @param array $where where-Klausel
	 * @global $ld->searchtitle
	 * @global $ld->searchurl
	 * @global $ld->searchdesc
	 * @global $ld->searchlongdesc
	 */
	private function getWhereSearchtextForLink($searchtext,&$where){
		if ( $searchtext ) {
			$whereSearch = array();
			if ($this->_ld->searchtitle){
				$whereSearch[] = "LOWER(#__ldlinks.title) LIKE '%$searchtext%'";
			}
			if ($this->_ld->searchurl){
				$whereSearch[] = "LOWER(#__ldlinks.url) LIKE '%$searchtext%'";
			}
			if ($this->_ld->searchdesc){
				$whereSearch[] = "LOWER(#__ldlinks.description) LIKE '%$searchtext%'";
			}
			if ($this->_ld->searchlongdesc){
				$whereSearch[] = "LOWER(#__ldlinks.description_long) LIKE '%$searchtext%'";
			}
			$this->convertWhereArrayToString($whereSearch,'OR',$where);
		}
	}
	
	// ===========================================================================
	// = Helper für den Zusammenbau von where-Klauseln                           = 
	// ===========================================================================

	/**
	 * Konvertiert das Array $whereSearch in ein String, indem es die einzelnen Elemente mit $op verknüft.
	 * Das Ergebnis wird dem Array $where hinzugefügt. 
	 *
	 * @param array  $whereSearch
	 * @param String $op AND oder OR
	 * @param array $where (wird verändert)
	 */
	private function convertWhereArrayToString(&$whereSearch,$op,&$where){
		$count=count($whereSearch);
		if ($count > 1) {
			$where[] = '(' . implode( ' '.$op.' ', $whereSearch ) .')';
		}
		elseif ($count == 1) {
			$where[] = $whereSearch[0];
		}
		elseif ($count == 0) {
			// $where[] = '1=0';
		}
	}
}

?>